@@ -9,14 +9,16 @@ import ConfigParser
 import shutil
 import subprocess
 import zipfile
+import codecs
 
 from burpui.misc.utils import human_readable as _hr
 from burpui.misc.backend.interface import BUIbackend, BUIserverException
 
 g_burpport = 4972
 g_burphost = '127.0.0.1'
-g_tmpdir   = '/tmp/buirestore'
+g_tmpdir   = u'/tmp/buirestore'
 g_burpbin  = '/usr/sbin/burp'
+g_stripbin = '/usr/sbin/vss_strip'
 
 class Burp(BUIbackend):
     states = {
@@ -60,61 +62,90 @@ class Burp(BUIbackend):
        ]
 
     def __init__(self, app=None, conf=None):
-        global g_burpport, g_burphost, g_tmpdir, g_burpbin
+        global g_burpport, g_burphost, g_tmpdir, g_burpbin, g_stripbin
         self.app = app
         self.host = g_burphost
         self.port = g_burpport
         self.burpbin = g_burpbin
+        self.stripbin = g_stripbin
         self.tmpdir = g_tmpdir
         self.running = []
         if conf:
-            config = ConfigParser.ConfigParser({'bport': g_burpport, 'tmpdir': g_tmpdir, 'burpbin': g_burpbin})
-            with open(conf) as fp:
+            config = ConfigParser.ConfigParser({'bport': g_burpport, 'bhost': g_burphost, 'tmpdir': g_tmpdir, 'burpbin': g_burpbin, 'stripbin': g_stripbin})
+            with codecs.open(conf, 'r', 'utf-8') as fp:
                 config.readfp(fp)
                 try:
                     self.port = config.getint('Burp1', 'bport')
+                    self.host = config.get('Burp1', 'bhost')
                     tdir = config.get('Burp1', 'tmpdir')
                     bbin = config.get('Burp1', 'burpbin')
+                    strip = config.get('Burp1', 'stripbin')
 
+                    if self.host not in ['127.0.0.1', '::1']:
+                        self.logger('warning', "Invalid value for 'bhost'. Must be '127.0.0.1' or '::1'. Falling back to '%s'", g_burphost)
+                        self.host = g_burphost
+
+                    if not strip.startswith('/'):
+                        self.logger('warning', "Please provide an absolute path for the 'stripbin' option. Fallback to '%s'", g_stripbin)
+                        strip = g_stripbin
+                    elif not re.match('^\S+$', strip):
+                        self.logger('warning', "Incorrect value for the 'stripbin' option. Fallback to '%s'", g_stripbin)
+                        strip = g_stripbin
+                    elif not os.path.isfile(strip) or not os.access(strip, os.X_OK):
+                        self.logger('warning', "'%s' does not exist or is not executable. Fallback to '%s'", strip, g_stripbin)
+                        strip = g_stripbin
+                        
                     if not bbin.startswith('/'):
-                        self.app.logger.warning('Please provide an absolute path for the \'burpbin\' option. Fallback to \'%s\'', g_burpbin)
+                        self.logger('warning', "Please provide an absolute path for the 'burpbin' option. Fallback to '%s'", g_burpbin)
                         bbin = g_burpbin
                     elif not re.match('^\S+$', bbin):
-                        self.app.logger.warning('Incorrect value for the \'burpbin\' option. Fallback to \'%s\'', g_burpbin)
+                        self.logger('warning', "Incorrect value for the 'burpbin' option. Fallback to '%s'", g_burpbin)
                         bbin = g_burpbin
                     elif not os.path.isfile(bbin) or not os.access(bbin, os.X_OK):
-                        self.app.logger.warning('\'%s\' does not exist or is not executable. Fallback to \'%s\'', bbin, g_burpbin)
+                        self.logger('warning', "'%s' does not exist or is not executable. Fallback to '%s'", bbin, g_burpbin)
                         bbin = g_burpbin
 
                     if not tdir.startswith('/'):
-                        self.app.logger.warning('Please provide an absolute path for the \'tmpdir\' option. Fallback to \'%s\'', g_tmpdir)
+                        self.logger('warning', "Please provide an absolute path for the 'tmpdir' option. Fallback to '%s'", g_tmpdir)
                         tdir = g_tmpdir
                     elif not re.match('^\S+$', tdir):
-                        self.app.logger.warning('Incorrect value for the \'tmpdir\' option. Fallback to \'%s\'', g_tmpdir)
+                        self.logger('warning', "Incorrect value for the 'tmpdir' option. Fallback to '%s'", g_tmpdir)
                         tdir = g_tmpdir
-                    elif os.path.isdir(tdir) and os.listdir(tdir):
-                        raise Exception('\'{0}\' is not empty!'.format(tdir))
+                    elif os.path.isdir(tdir) and os.listdir(tdir) and not self.app.config.get('TESTING'):
+                        raise Exception("'{0}' is not empty!".format(tdir))
                     elif os.path.isdir(tdir) and not os.access(tdir, os.W_OK|os.X_OK):
-                        self.app.logger.warning('\'%s\' is not writable. Fallback to \'%s\'', tdir, g_tmpdir)
+                        self.logger('warning', "'%s' is not writable. Fallback to '%s'", tdir, g_tmpdir)
                         tdir = g_tmpdir
 
                     self.burpbin = bbin
                     self.tmpdir = tdir
+                    self.stripbin = strip
                 except ConfigParser.NoOptionError, e:
-                    self.app.logger.error(str(e))
+                    self.logger('error', str(e))
                 except ConfigParser.NoSectionError, e:
-                    self.app.logger.error(str(e))
+                    self.logger('error', str(e))
 
-        self.app.logger.info('burp port: %d', self.port)
-        self.app.logger.info('burp host: %s', self.host)
-        self.app.logger.info('burp binary: %s', self.burpbin)
-        self.app.logger.info('temporary dir: %s', self.tmpdir)
+        self.logger('info', 'burp port: %d', self.port)
+        self.logger('info', 'burp host: %s', self.host)
+        self.logger('info', 'burp binary: %s', self.burpbin)
+        self.logger('info', 'strip binary: %s', self.stripbin)
+        self.logger('info', 'temporary dir: %s', self.tmpdir)
 
+    def logger(self, level, *args):
+        if self.app:
+            logs = {
+                'info': self.app.logger.info,
+                'error': self.app.logger.error,
+                'debug': self.app.logger.debug,
+                'warning': self.app.logger.warning
+            }
+            if level in logs:
+                logs[level](*args)
     """
     Utilities functions
     """
 
-    def status(self, query='\n'):
+    def status(self, query='\n', agent=None):
         """
         status connects to the burp status port, ask the given 'question' and
         parses the output in an array
@@ -144,10 +175,10 @@ class Burp(BUIbackend):
             f.close()
             return r
         except socket.error:
-            self.app.logger.error('Cannot contact burp server at %s:%s', self.host, self.port)
+            self.logger('error', 'Cannot contact burp server at %s:%s', self.host, self.port)
             raise BUIserverException('Cannot contact burp server at {0}:{1}'.format(self.host, self.port))
 
-    def parse_backup_log(self, f, n, c=None):
+    def parse_backup_log(self, f, n, c=None, agent=None):
         """
         parse_backup_log parses the log.gz of a given backup and returns a dict
         containing different stats used to render the charts in the reporting view
@@ -225,7 +256,7 @@ class Burp(BUIbackend):
             for key, regex in lookup_complex.iteritems():
                 r = re.search(regex, line)
                 if r:
-                    self.app.logger.debug("match[1]: '{0}'".format(r.group(1)))
+                    self.logger('debug', "match[1]: '{0}'".format(r.group(1)))
                     sp = re.split('\s+', r.group(1))
                     backup[key] = {
                             'new':       int(sp[0]),
@@ -238,24 +269,28 @@ class Burp(BUIbackend):
                     break
         return backup
 
-    def get_counters(self, name=None):
+    def get_counters(self, name=None, agent=None):
         """
         get_counters parses the stats of the live status for a given client and
         returns a dict
         """
         r = {}
-        if not name or name not in self.running:
-            return r
+        if agent:
+            if not name or name not in self.running[agent]:
+                return r
+        else:
+            if not name or name not in self.running:
+                return r
         f = self.status('c:{0}\n'.format(name))
         if not f:
             return r
         for line in f:
-            self.app.logger.debug('line: {0}'.format(line))
+            self.logger('debug', 'line: {0}'.format(line))
             rs = re.search('^{0}\s+(\d)\s+(\S)\s+(.+)$'.format(name), line)
             if rs and rs.group(2) == 'r' and int(rs.group(1)) == 2:
                 c = 0
                 for v in rs.group(3).split('\t'):
-                    self.app.logger.debug('{0}: {1}'.format(self.counters[c], v))
+                    self.logger('debug', '{0}: {1}'.format(self.counters[c], v))
                     if c > 0 and c < 15:
                         val = map(int, v.split('/'))
                         if val[0] > 0 or val[1] > 0 or val[2] or val[3] > 0:
@@ -279,7 +314,7 @@ class Burp(BUIbackend):
             r['timeleft'] = -1
         return r
 
-    def is_backup_running(self, name=None):
+    def is_backup_running(self, name=None, agent=None):
         """
         is_backup_running returns True if the given client is currently running a
         backup
@@ -296,7 +331,7 @@ class Burp(BUIbackend):
                 return True
         return False
 
-    def is_one_backup_running(self):
+    def is_one_backup_running(self, agent=None):
         """
         is_one_backup_running returns a list of clients name that are currently
         running a backup
@@ -307,12 +342,12 @@ class Burp(BUIbackend):
         except BUIserverException:
             return r
         for c in cls:
-            if self.is_backup_running(c['name']):
+            if self.is_backup_running(c['name'], agent):
                 r.append(c['name'])
         self.running = r
         return r
 
-    def get_all_clients(self):
+    def get_all_clients(self, agent=None):
         """
         get_all_clients returns a list of dict representing each clients with their
         name, state and last backup date
@@ -320,14 +355,14 @@ class Burp(BUIbackend):
         j = []
         f = self.status()
         for line in f:
-            self.app.logger.debug("line: '{0}'".format(line))
+            self.logger('debug', "line: '{0}'".format(line))
             regex = re.compile('\s*(\S+)\s+\d\s+(\S)\s+(.+)')
             m = regex.search(line)
             c = {}
             c['name'] = m.group(1)
             c['state'] = self.states[m.group(2)]
             infos = m.group(3)
-            self.app.logger.debug("infos: '{0}'".format(infos))
+            self.logger('debug', "infos: '{0}'".format(infos))
             if infos == "0":
                 c['last'] = 'never'
             elif re.match('^\d+\s\d+\s\d+$', infos):
@@ -339,7 +374,7 @@ class Burp(BUIbackend):
             j.append(c)
         return j
 
-    def get_client(self, name=None):
+    def get_client(self, name=None, agent=None):
         """
         get_client returns a list of dict representing the backups (with its number
         and date) of a given client
@@ -352,7 +387,7 @@ class Burp(BUIbackend):
         for line in f:
             if not re.match('^{0}\t'.format(c), line):
                 continue
-            self.app.logger.debug("line: '{0}'".format(line))
+            self.logger('debug', "line: '{0}'".format(line))
             regex = re.compile('\s*(\S+)\s+\d\s+(\S)\s+(.+)')
             m = regex.search(line)
             if m.group(3) == "0" or m.group(2) not in [ 'i', 'c', 'C' ]:
@@ -368,7 +403,7 @@ class Burp(BUIbackend):
         r.reverse()
         return r
 
-    def get_tree(self, name=None, backup=None, root=None):
+    def get_tree(self, name=None, backup=None, root=None, agent=None):
         """
         get_tree returns a list of dict representing files/dir (with their attr)
         within a given path
@@ -384,7 +419,7 @@ class Burp(BUIbackend):
         f = self.status('c:{0}:b:{1}:p:{2}\n'.format(name, backup, top))
         useful = False
         for line in f:
-            self.app.logger.debug("line: '{0}'".format(line))
+            self.logger('debug', "line: '{0}'".format(line))
             if not useful and re.match('^-list begin-$', line):
                 useful = True
                 continue
@@ -399,19 +434,19 @@ class Burp(BUIbackend):
                         t['type'] = 'd'
                     else:
                         t['type'] = 'f'
-                    sp = re.split('\s+', line)
+                    sp = re.split('\s+', line, 7)
                     t['mode'] = sp[0]
                     t['inodes'] = sp[1]
                     t['uid'] = sp[2]
                     t['gid'] = sp[3]
                     t['size'] = '{0:.1eM}'.format(_hr(sp[4]))
                     t['date'] = '{0} {1}'.format(sp[5], sp[6])
-                    t['name'] = sp[len(sp)-1]
+                    t['name'] = sp[7]
                     t['parent'] = top
                     r.append(t)
         return r
 
-    def restore_files(self, name=None, backup=None, files=None):
+    def restore_files(self, name=None, backup=None, files=None, agent=None):
         if not name or not backup or not files:
             return None
         flist = json.loads(files)
@@ -419,27 +454,46 @@ class Burp(BUIbackend):
             return None
         if os.path.isdir(self.tmpdir):
             shutil.rmtree(self.tmpdir)
+        full_reg = u''
         for r in flist['restore']:
-            reg = ''
+            reg = u''
             if r['folder'] and r['key'] != '/':
-                reg = r['key']+'/'
+                reg += '^'+r['key']+'/|'
             else:
-                reg = r['key']
-            #cmd = self.burpbin+' -C '+name+' -a r -b '+str(backup)+' -r \''+reg+'\' -d '+self.tmpdir
-            status = subprocess.call([self.burpbin, '-C', name, '-a', 'r', '-b', str(backup), '-r', reg, '-d', self.tmpdir])
-            if status != 0:
-                return None
+                reg += '^'+r['key']+'$|'
+            full_reg += reg
+
+        cmd = [self.burpbin, '-C', name, '-a', 'r', '-b', str(backup), '-r', full_reg.rstrip('|'), '-d', self.tmpdir]
+        self.logger('debug', cmd)
+        status = subprocess.call(cmd)
+        if status != 0:
+            return None
 
         zip_dir = self.tmpdir.rstrip(os.sep)
         zip_file = zip_dir+'.zip'
         if os.path.isfile(zip_file):
             os.remove(zip_file)
         zip_len = len(zip_dir) + 1
+        stripping = True
         with zipfile.ZipFile(zip_file, mode='w', compression=zipfile.ZIP_DEFLATED) as zf:
             for dirname, subdirs, files in os.walk(zip_dir):
                 for filename in files:
                     path = os.path.join(dirname, filename)
+                    if stripping and os.path.isfile(path):
+                        self.logger('debug', "stripping file: %s", path)
+                        shutil.move(path, path+'.tmp')
+                        status = subprocess.call([self.stripbin, '-i', path+'.tmp', '-o', path])
+                        if status != 0:
+                            os.remove(path)
+                            shutil.move(path+'.tmp', path)
+                            stripping = False
+                            self.logger('debug', "Disable stripping since this file does not seem to embed VSS headers")
+                        else:
+                            os.remove(path+'.tmp')
+
                     entry = path[zip_len:]
                     zf.write(path, entry)
+
+        shutil.rmtree(self.tmpdir)
         return zip_file
 
