@@ -33,7 +33,7 @@
 	 */
 	$("#tree").fancytree({
 		checkbox: true,
-		selectMode: 2,
+		selectMode: 3,
 		extensions: ["glyph", "table", "gridnav", "filter"],
 		glyph: {
 			map: {
@@ -50,13 +50,13 @@
 				// expanderOpen: "glyphicon glyphicon-collapse-down",
 				folder: "glyphicon glyphicon-folder-close",
 				folderOpen: "glyphicon glyphicon-folder-open",
-				loading: "glyphicon glyphicon-refresh"
+				loading: "icon-refresh-animate glyphicon glyphicon-refresh"
 				// loading: "icon-spinner icon-spin"
 			}
 		},
 		source: function() { 
 			r = [];
-			$.getJSON('{{ url_for("client_tree", name=cname, backup=nbackup) }}', function(data) {
+			$.getJSON('{{ url_for("client_tree", name=cname, backup=nbackup, server=server) }}', function(data) {
 				if (!data.results) {
 					if (data.notif) {
 						$.each(data.notif, function(i, n) {
@@ -72,6 +72,8 @@
 					r.push(s);
 				});
 			});
+			$("#waiting-container").hide();
+			$("#tree-container").show();
 			return r;
 		},
 		lazyLoad: function(event, data) {
@@ -82,7 +84,7 @@
 			r = [];
 			p = node.key;
 			if (p !== "/") p += '/';
-			$.getJSON('{{ url_for("client_tree", name=cname, backup=nbackup) }}?root='+p, function(data) {
+			$.getJSON('{{ url_for("client_tree", name=cname, backup=nbackup, server=server) }}?root='+p, function(data) {
 				if (!data.results) {
 					if (data.notif) {
 						$.each(data.notif, function(i, n) {
@@ -202,4 +204,10 @@
 		});
 		e.preventDefault();
 	});
+	$("#btn-clear").on('click', function() {
+		tree.visit(function(node){
+			node.setSelected(false);
+		});
+		return false;
+	});
 
